from django.template import Library, Node, Variable, VariableDoesNotExist
from django.template.loader import get_template


register = Library()

# Fix django templatetags module loader
__path__ = ""

class AdrestInclusionNode(Node):

    def __init__(self, name, *args):
        self.var = Variable(name)
        super(AdrestInclusionNode, self).__init__()

    def render(self, context):
        try:
            target = self.var.resolve(context)
            assert target
        except (VariableDoesNotExist, AssertionError):
            return ''

        emitter = context.get('emitter')
        t_name = emitter.get_template_path(target)
        t = get_template(t_name)
        context.dicts.append(dict(content=target))
        response = t.nodelist.render(context)
        context.pop()
        return response


def adrest_include(parser, token):
    bits = token.split_contents()[1:]
    return AdrestInclusionNode(*bits)
adrest_include = register.tag(adrest_include)



from . import status as http_status


class HttpError(Exception):
    """ Represent Http Error.
    """
    def __init__(self, message, status=http_status.HTTP_400_BAD_REQUEST):
        self.message, self.status = message, status
        super(HttpError, self).__init__(message)

    def __str__(self):
        return self.message

    __repr__ = __str__



from django.utils import simplejson as json

from . import status
from .exceptions import HttpError


class BaseParser(object):

    media_type = None

    def __init__(self, resource):
        self.resource = resource

    @staticmethod
    def parse(request):
        return request.raw_post_data


class JSONParser(BaseParser):

    media_type = 'application/json'

    @staticmethod
    def parse(request):
        try:
            return json.loads(request.raw_post_data)
        except ValueError, e:
            raise HttpError('JSON parse error - %s' % str(e), status=status.HTTP_400_BAD_REQUEST)


class XMLParser(BaseParser):

    media_type = 'application/xml'


class FormParser(BaseParser):

    media_type = 'application/x-www-form-urlencoded'

    @staticmethod
    def parse(request):
        source = dict(request.REQUEST.iteritems())
        # Fix Django dont parse PUT request from form-data
        if request.method == "PUT":
            request.method = "POST"
            del request._files
            request._load_post_and_files()
            request.method = "PUT"
            source.update(dict(request.POST.iteritems()))
        return source



#!/usr/bin/env python
"""Settings to run library tests
"""
import sys
from os.path import dirname, abspath

from django.conf import settings as django_settings

from tests import settings


if not django_settings.configured:
    django_settings.configure(
            **dict([(x, getattr(settings, x)) for x in dir(settings) if x.isupper()]))


def run_tests(*test_args):
    # Import django tests runner
    from django.test.simple import DjangoTestSuiteRunner
    if not test_args:
        test_args = ['main', 'simple']
    parent = dirname(abspath(__file__))
    sys.path.insert(0, parent)
    tests_runner = DjangoTestSuiteRunner(verbosity=1, interactive=True, failfast=True)
    failures = tests_runner.run_tests(test_args)
    sys.exit(failures)

if __name__ == '__main__':
    run_tests(*sys.argv[1:])



from django.core.serializers import get_serializer
from django.core.serializers.json import DjangoJSONEncoder
from django.db.models import Model
from django.db.models.query import QuerySet
from django.http import HttpResponse
from django.utils.encoding import smart_unicode
from django.utils.functional import curry
from django.utils.simplejson import dumps, loads

from .paginator import Paginator


class JSONEncoder(DjangoJSONEncoder):

    def __init__(self, *args, **kwargs):
        self.serializer = get_serializer('json')()
        self.serializer.use_natural_keys = False
        super(JSONEncoder, self).__init__(*args, **kwargs)

    def handle_fk_field(self, o, field):
        """ Handle field, without base requests.
        """
        key = getattr(o, field.column, None)
        if not key is None:
            self.serializer._current[field.name] = key
        return key

    def default(self, o):

        if isinstance(o, set):
            return list(o)

        if isinstance(o, QuerySet):
            return list(o)

        if isinstance(o, HttpResponse):
            return o.content

        if isinstance(o, Model):
            self.serializer.start_object(o)
            for field in o._meta.local_fields:
                if field.serialize:
                    if field.rel is None:
                        self.serializer.handle_field(o, field)
                    else:
                        self.handle_fk_field(o, field)
            for field in o._meta.many_to_many:
                if field.serialize:
                    self.serializer.handle_m2m_field(o, field)
            return {
                "model"  : smart_unicode(o._meta),
                "pk"     : smart_unicode(o._get_pk_val(), strings_only=True),
                "fields" : self.serializer._current
            }

        elif isinstance(o, Paginator):
            return dict(
                count = o.count,
                page = o.page.number,
                next = o.next,
                prev = o.previous,
                resources = o.resources,
            )

        return super(JSONEncoder, self).default(o)


json_dumps = curry(dumps, cls=JSONEncoder)


def xml_dumps(o):
    simple = loads(json_dumps(o))
    return ''.join(_xml_dump(simple))

def _xml_dump(o):
    tag = it = None

    if isinstance(o, list):
        tag = 'items'
        it = iter(o)

    elif isinstance(o, dict) and o.has_key('model'):
        tag = o.get('model').split('.')[1]
        it = o.iteritems()

    elif isinstance(o, dict):
        it = o.iteritems()

    elif isinstance(o, tuple):
        tag = str(o[0])
        it = (i for i in o[1:])

    else:
        yield str(o)

    if tag:
        yield "<%s>" % tag

    if it:
        try:
            while True:
                v = next(it)
                yield ''.join(_xml_dump(v))
        except StopIteration:
            pass

    if tag:
        yield "</%s>" % tag



from .models import Book
from adrest.views import ResourceView


class AuthorResource(ResourceView):
    allowed_methods = 'GET', 'POST'
    model = 'main.author'
    url_regex = '^owner/$'


class BookResource(ResourceView):
    allowed_methods = 'GET', 'POST', 'PUT', 'DELETE'
    model = 'main.book'
    parent = AuthorResource


class BookPrefixResource(BookResource):
    prefix = 'test'


class ArticleResource(ResourceView):
    allowed_methods = 'GET', 'PUT', 'DELETE'
    model = 'main.article'
    parent = BookPrefixResource

    def put(self, request, **kwargs):
        assert False, "Assertion error"

    def delete(self, request, **kwargs):
        raise Exception("Some error")


class OtherResource(ResourceView):
    parent = BookResource
    url_prefix = 'other'

    def get(self, request, **kwargs):
        return True


class SomeOtherResource(ResourceView):
    parent = AuthorResource
    url_params = 'device',

    def get(self, request, **kwargs):
        return True


class CustomResource(ResourceView):
    model = 'main.book'
    queryset = Book.objects.all()
    template = 'main/custom.xml'

    def get(self, request, **kwargs):
        return list(self.queryset)



#!/usr/bin/env python
# coding: utf-8
import sys
import traceback
from logging import getLogger

from django.conf.urls.defaults import url
from django.core.exceptions import ObjectDoesNotExist, MultipleObjectsReturned, ValidationError
from django.core.mail import mail_admins
from django.http import HttpResponse
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import View

from .utils import status, MetaOptions, gen_url_name, gen_url_regex
from .utils.exceptions import HttpError
from .utils.paginator import Paginator
from .utils.tools import as_tuple
from adrest import settings
from adrest.mixin import auth, emitter, handler, parser, throttle
from adrest.signals import api_request_started, api_request_finished


logger = getLogger('django.request')


class ResourceMetaClass(handler.HandlerMeta, throttle.ThrottleMeta, emitter.EmitterMeta,
        parser.ParserMeta, auth.AuthMeta):
    """ MetaClass for ResourceView.
        Create meta options.
    """

    def __new__(mcs, name, bases, params):

        # Create meta if not exists
        params['meta'] = params.get('meta', MetaOptions())

        # Run other meta classes
        cls = super(ResourceMetaClass, mcs).__new__(mcs, name, bases, params)

        # Prepare allowed methods
        cls.allowed_methods = mcs.prepare_methods(cls.allowed_methods)

        # Check parent
        if cls.parent:
            try:
                cls.meta.parents += cls.parent.meta.parents + [cls.parent]
            except AttributeError:
                raise TypeError("%s.parent must be instance of %s" % (name, "ResourceView"))

        # Meta name (maybe precalculate in handler)
        cls.meta.name = cls.meta.name or cls.name or ''.join(bit for bit in name.split('Resource') if bit).lower()

        # Prepare urls
        cls.url_params = list(as_tuple(cls.url_params))
        cls.meta.url_name = cls.url_name or '-'.join(gen_url_name(cls))
        cls.meta.url_regex = cls.url_regex or '/'.join(gen_url_regex(cls))

        return cls

    @staticmethod
    def prepare_methods(methods):
        " Prepare allowed methods. "

        methods = as_tuple(methods)

        if not 'OPTIONS' in methods and settings.ALLOW_OPTIONS:
            methods += 'OPTIONS',

        if not 'HEAD' in methods and 'GET' in methods:
            methods += 'HEAD',

        return methods


class ResourceView(handler.HandlerMixin,
        throttle.ThrottleMixin,
        emitter.EmitterMixin,
        parser.ParserMixin,
        auth.AuthMixin,
        View):

    # Create meta options
    __metaclass__ = ResourceMetaClass

    # Allowed methods
    allowed_methods = 'GET',

    # Name (Defaul by model name or class name)
    name = None

    # Link to api if connected
    api = None

    # Saves access log if enabled
    log = True

    # Link to parent resource
    parent = None

    # URL name and regex prefix
    prefix = ''

    # Some custom URI params here
    url_params = None
    url_regex = None
    url_name = None

    # If children object in hierarchy has FK=Null to parent, allow to get this
    # object (default: True)
    allow_public_access = False

    @csrf_exempt
    def dispatch(self, request, **resources):

        # Send started signal
        api_request_started.send(self, request = request)

        # Current HTTP method
        method = request.method.upper()

        try:

            # Check request method
            self.check_method_allowed(method)

            # Authentificate
            # We do not restrict access for OPTIONS request.
            if method == 'OPTIONS' and settings.ALLOW_OPTIONS:
                self.identifier = 'anonymous'
            else:
                self.identifier = self.authenticate(request)

            # Throttle check
            self.throttle_check()

            # Get required resources
            resources = self.get_resources(request, resource=self, **resources)

            # Check owners
            self.check_owners(**resources)

            # Check rights for resources with this method
            self.check_rights(resources, request=request)

            # Parse content
            if method in ('POST', 'PUT'):
                request.data = self.parse(request)
                if isinstance(request.data, basestring):
                    request.data = dict()

            # Get the appropriate create/read/update/delete function
            func = getattr(self, self.callmap.get(method, None))

            # Get function data
            content = func(request, **resources)

            # Serialize response
            response = self.emit(content, request=request)
            try:
                response["Allow"] = ', '.join(self.allowed_methods),
                response["Vary"] = 'Authenticate, Accept'

                # Add pagination headers
                # http://www.w3.org/Protocols/9707-link-header.html
                if isinstance(content, Paginator):
                    linked_resources = []
                    if content.next:
                        linked_resources.append('<%s>; rel="next"' % content.next)
                    if content.previous:
                        linked_resources.append('<%s>; rel="previous"' % content.previous)
                    response["Link"] = ", ".join(linked_resources)

            except TypeError:
                raise ValueError("Emitter must return HttpResponse")

        except (HttpError, AssertionError, ValidationError), e:
            response = HttpResponse(unicode(e), status=getattr(e, 'status', status.HTTP_400_BAD_REQUEST))
            response = self.emit(response, request=request)

        except Exception, e:
            response = self.handle_exception(e, request=request)

        errors_mail(response, request)

        # Send finished signal
        api_request_finished.send(self, request=request, response=response, **resources)

        return response

    @classmethod
    def check_method_allowed(cls, method):
        """ Ensure the request HTTP method is permitted for this resource, raising a ResourceException if it is not.
        """
        if not method in cls.callmap.keys():
            raise HttpError('Unknown or unsupported method \'%s\'' % method, status=status.HTTP_501_NOT_IMPLEMENTED)

        if not method in cls.allowed_methods:
            raise HttpError('Method \'%s\' not allowed on this resource.' % method, status=status.HTTP_405_METHOD_NOT_ALLOWED)

    @classmethod
    def get_resources(cls, request, resource=None, **resources):
        " Parse resource objects from URL and GET. "

        # Get from parent
        if cls.parent:
            resources = cls.parent.get_resources(request, resource=resource, **resources)

        pk = resources.get(cls.meta.name) or request.GET.get(cls.meta.name)
        if not cls.model or not pk:
            return resources

        try:
            resources[cls.meta.name] = cls.model.objects.get(pk = pk)

        except (ObjectDoesNotExist, ValueError):
            raise HttpError("Resource not found.", status=status.HTTP_404_NOT_FOUND)

        except MultipleObjectsReturned:
            raise HttpError("Resources conflict.", status=status.HTTP_409_CONFLICT)

        return resources

    @classmethod
    def check_owners(cls, **resources):
        """ Recursive scanning of the fact that the child has FK
            to the parent and in resources we have right objects.

            We check that in request like /author/1/book/2/page/3

            Page object with pk=3 has ForeignKey field linked to Book object with pk=2
            and Book with pk=2 has ForeignKey field linked to Author object with pk=1.
        """

        if cls.allow_public_access or not cls.parent:
            return True

        cls.parent.check_owners(**resources)

        resource = resources.get(cls.meta.name)
        if cls.model and cls.parent.model and resource:
            parent_resource = resources.get(cls.parent.meta.name)
            parent_resource_id = getattr(resource, "%s_id" % cls.parent.meta.name, None)
            try:
                assert parent_resource and parent_resource.pk == parent_resource_id
            except AssertionError:
                # 403 Error if there is error in parent-children relationship
                raise HttpError("Access forbidden.", status=status.HTTP_403_FORBIDDEN)

        return True

    @staticmethod
    def handle_exception(e, request=None, exc_info=None):
        """ Handle code exception.
        """
        if settings.DEBUG:
            raise

        logger.warning('ADREST API Error: %s' % request.path,
            exc_info=exc_info
        )

        return HttpResponse(str(e), status=500)

    @property
    def version(self):
        return str(self.api or '')

    @classmethod
    def as_url(cls, api=None, name_prefix='', url_prefix=''):
        " Generate url for resource. "
        url_prefix = url_prefix and "%s/" % url_prefix
        name_prefix = name_prefix and "%s-" % name_prefix

        url_regex = '^%s%s$' % (url_prefix, cls.meta.url_regex.lstrip('^').rstrip('/$'))
        url_regex = url_regex.replace('//', '/')
        url_name = '%s%s' % (name_prefix, cls.meta.url_name)

        return url(url_regex, cls.as_view(api=api), name = url_name)


def errors_mail(response, request):

    if not response.status_code in settings.MAIL_ERRORS:
        return False

    subject = 'ADREST API Error (%s): %s' % (response.status_code, request.path)
    stack_trace = '\n'.join(traceback.format_exception(*sys.exc_info()))
    message = """
Stacktrace:
===========
%s

Handler data:
=============
%s

Request information:
====================
%s

""" % (stack_trace, repr(getattr(request, 'data', None)), repr(request))
    return mail_admins(subject, message, fail_silently=True)



from adrest.utils import status
from adrest.utils.auth import AnonimousAuthenticator, BaseAuthenticator
from adrest.utils.exceptions import HttpError
from adrest.utils.tools import as_tuple


def check_authenticators(authenticators):
    authenticators = as_tuple(authenticators)
    for a in authenticators:
        assert issubclass(a, BaseAuthenticator), "Authenticators must be subclasses of BaseAuthenticator"
    return authenticators


class AuthMeta(type):

    def __new__(mcs, name, bases, params):
        cls = super(AuthMeta, mcs).__new__(mcs, name, bases, params)
        cls.authenticators = check_authenticators(cls.authenticators)
        return cls


class AuthMixin(object):
    " Adds pluggable authentication behaviour "

    __metaclass__ = AuthMeta

    authenticators = AnonimousAuthenticator
    identifier = ''
    auth = None

    def authenticate(self, request):
        """ Attempt to authenticate the request, returning an authentication context or None.
            An authentication context may be any object, although in many cases it will simply be a :class:`User` instance.
        """
        for authenticator in self.authenticators:
            auth = authenticator(self)
            result = auth.authenticate(request)
            if result:
                self.auth = auth
                return result
        else:
            raise HttpError("Authorization required.", status=status.HTTP_401_UNAUTHORIZED)

        return True

    def check_rights(self, resources, request=None):
        " Check rights of client for queried resources "

        if not self.auth:
            return True
        try:
            assert self.auth.test_rights(resources, request=request)
        except AssertionError:
            raise HttpError("Access forbidden.", status=status.HTTP_403_FORBIDDEN)



from django.forms.models import ModelChoiceField
from django.utils.encoding import smart_unicode

from .utils.auth import AnonimousAuthenticator
from .utils.emitter import HTMLTemplateEmitter, JSONEmitter
from .views import ResourceView


class MapResource(ResourceView):
    " Simple Api Map. "

    log = False
    emitters = HTMLTemplateEmitter, JSONEmitter
    authenticators = AnonimousAuthenticator
    template = 'api/map.html'

    url_regex = r'^map$'

    def get(self, *args, **Kwargs):
        return self.api.str_version, list(self.gen_apimap())

    def gen_apimap(self):
        for url_name in sorted(self.api.resources.iterkeys()):
            resource = self.api.resources[url_name]
            info = dict(
                name = url_name,
                methods = resource.allowed_methods,
                fields = []
            )
            if resource.model:
                info['resource'] = resource.model.__name__

            models = [p.model for p in resource.meta.parents if p.model]

            if resource.form and ('POST' in resource.allowed_methods or 'PUT' in resource.allowed_methods):
                info['fields'] += [
                    (name, dict(
                        required = f.required and f.initial is None,
                        help = smart_unicode(f.help_text + ''))
                    )
                    for name, f in resource.form.base_fields.iteritems()
                    if not (isinstance(f, ModelChoiceField) and f.choices.queryset.model in models)
                ]

            for a in resource.authenticators:
                info['fields'] += a.get_fields()

            info['auth'] = set(map(str, resource.authenticators))
            key = resource.meta.url_regex.replace("(?P", "").replace("[^/]+)", "").replace("?:", "").replace("$", "")
            yield key, info



from django.db.models import get_model, Model
from django.http import HttpResponse

from adrest.forms import PartitialForm
from adrest.settings import LIMIT_PER_PAGE
from adrest.utils import status, MetaOptions
from adrest.utils.exceptions import HttpError
from adrest.utils.paginator import Paginator


class HandlerMeta(type):
    def __new__(mcs, name, bases, params):

        params['meta'] = params.get('meta', MetaOptions())
        cls = super(HandlerMeta, mcs).__new__(mcs, name, bases, params)

        # Create model from string
        if isinstance(cls.model, basestring):
            assert '.' in cls.model, ("'model_class' must be either a model"
                                    " or a model name in the format"
                                    " app_label.model_name")
            cls.model = get_model(*cls.model.split("."))

        if cls.model:
            assert issubclass(cls.model, Model), "'model' attribute must be subclass of Model "
            cls.meta.name = cls.model._meta.module_name
            cls.meta.model_fields = set(f.name for f in cls.model._meta.fields)
            if cls.queryset is None:
                cls.queryset = cls.model.objects.all()

        # Create form if not exist
        if cls.model and not cls.form:
            class DynForm(PartitialForm):
                class Meta():
                    model = cls.model
                    fields = cls.form_fields
                    exclude = cls.form_exclude
            cls.form = DynForm

        return cls


class HandlerMixin(object):

    __metaclass__ = HandlerMeta

    limit_per_page = LIMIT_PER_PAGE
    model = None
    queryset = None
    form = None
    form_fields = None
    form_exclude = None
    callmap = { 'GET': 'get', 'POST': 'post',
                'PUT': 'put', 'DELETE': 'delete',
                'OPTIONS': 'options', 'HEAD': 'head' }

    def __init__(self, *args, **kwargs):
        super(HandlerMixin, self).__init__(*args, **kwargs)
        # Copy self queryset for prevent query caching
        if not self.queryset is None:
            self.queryset = self.queryset.all()

    @staticmethod
    def head(*args, **kwargs):
        return HttpResponse()

    def get(self, request, **resources):
        assert self.model, "This auto method required in model."
        instance = resources.get(self.meta.name)
        if instance:
            return instance

        return self.paginate(request, self.get_queryset(request, **resources))

    def post(self, request, **resources):
        form = self.form(data=request.data, **resources)
        if form.is_valid():
            return form.save()
        raise HttpError(form.errors.as_text(), status=status.HTTP_400_BAD_REQUEST)

    def put(self, request, **resources):
        instance = resources.get(self.meta.name)
        if not instance:
            raise HttpError("Bad request", status=status.HTTP_404_NOT_FOUND)

        form = self.form(data=request.data, instance=instance, **resources)
        if form.is_valid():
            return form.save()

        raise HttpError(form.errors.as_text(), status=status.HTTP_400_BAD_REQUEST)

    def delete(self, request, **resources):
        instance = resources.get(self.meta.name)
        if not instance:
            raise HttpError("Bad request", status=status.HTTP_400_BAD_REQUEST)

        for name, owner in resources.items():
            if hasattr(instance, '%s_id' % name):
                assert owner.pk == getattr(instance, '%s_id' % name)
        instance.delete()
        return HttpResponse("%s has been deleted." % self.meta.name.capitalize())

    @staticmethod
    def options(request, **kwargs):
        return 'OK'

    def get_queryset(self, request, **resources):

        if self.queryset is None:
            return None

        # Make filters from URL variables or resources
        filters = dict((k, v) for k, v in resources.iteritems() if k in self.meta.model_fields)

        # Make filters from GET variables
        for field in request.GET.iterkeys():
            if not field in self.meta.model_fields or filters.has_key(field):
                continue
            converter = self.model._meta.get_field(field).to_python
            filters[field] = map(converter, request.GET.getlist(field))

        filters = dict(
            (k, v) if not isinstance(v, (list, tuple))
            else ("%s__in" % k, v) if len(v) > 1
            else (k, v[0])
            for k, v in filters.iteritems()
        )

        return self.queryset.filter(**filters)

    def paginate(self, request, qs):
        """ Paginate queryset.
        """
        return Paginator(request, qs, self.limit_per_page)



class MetaOptions(object):
    " Resource meta options "

    def __init__(self):
        self.name = ''
        self.url_name = ''
        self.url_regex = ''
        self.parents = []
        self.model_fields = set()

        self.emitters_dict = dict()
        self.emitters_types = []
        self.default_emitter = None

        self.parsers_dict = dict()
        self.default_parser = None

    def __str__(self):
        return "%(url_name)s(%(name)s) - %(url_regex)s %(parents)s" % self.__dict__

    __repr__ = __str__


def gen_url_name(resource):
    if resource.parent:
        yield resource.parent.meta.url_name

    if resource.prefix:
        yield resource.prefix

    for p in resource.url_params:
        yield p

    yield resource.meta.name


def gen_url_regex(resource):
    for r in resource.meta.parents:
        if r.url_regex:
            yield r.url_regex.rstrip('/$').lstrip('^')
        else:
            yield '%(name)s/(?P<%(name)s>[^/]+)' % dict(
                    name = r.meta.name)

    for p in resource.url_params:
        yield '%(name)s/(?P<%(name)s>[^/]+)' % dict(name = p)

    if resource.prefix:
        yield resource.prefix

    yield '%(name)s/(?:(?P<%(name)s>[^/]+)/)?' % dict(name = resource.meta.name)



from adrest.utils.parser import FormParser, XMLParser, JSONParser, BaseParser
from adrest.utils import MetaOptions
from adrest.utils.tools import as_tuple


class ParserMeta(type):

    def __new__(mcs, name, bases, params):
        params['meta'] = params.get('meta', MetaOptions())
        cls = super(ParserMeta, mcs).__new__(mcs, name, bases, params)
        cls.parsers = as_tuple(cls.parsers)
        cls.meta.default_parser = cls.parsers[0] if cls.parsers else None

        for p in cls.parsers:
            assert issubclass(p, BaseParser), "Parser must be subclass of BaseParser"
            cls.meta.parsers_dict[p.media_type] = p

        return cls


class ParserMixin(object):

    __metaclass__ = ParserMeta

    parsers = FormParser, XMLParser, JSONParser

    def parse(self, request):
        " Parse request content "

        content_type = self.determine_content(request)
        if content_type:
            split = content_type.split(';', 1)
            if len(split) > 1:
                content_type = split[0]
            content_type = content_type.strip()

        parser = self.meta.parsers_dict.get(content_type, self.meta.default_parser)
        return parser(self).parse(request)

    @staticmethod
    def determine_content(request):
        " Determine request content "

        if not request.META.get('CONTENT_LENGTH', None) and not request.META.get('TRANSFER_ENCODING', None):
            return None

        return request.META.get('CONTENT_TYPE', None)



from datetime import datetime
from time import mktime

from django.template import RequestContext, loader
from django.http import HttpResponse
from django.db.models.base import ModelBase, Model
from os import path as op

from .paginator import Paginator
from .serializer import json_dumps, xml_dumps
from adrest.utils.status import HTTP_200_OK


class BaseEmitter(object):

    media_type = None

    def __init__(self, resource):
        self.resource = resource
        self.ext = str(self.media_type).split('/')[-1]

    def emit(self, content, request=None):
        if not isinstance(content, HttpResponse):
            return HttpResponse(self.serialize(content, request=request),
                    mimetype=self.media_type,
                    status=HTTP_200_OK)

        response = content
        response.content = self.serialize(content, request=request)
        return response

    @staticmethod
    def serialize(content, request=None):
        return content


class TemplateEmitter(BaseEmitter):
    """ All emitters must extend this class, set the media_type attribute, and
        override the emit() function.
    """
    def serialize(self, content, request=None):

        if isinstance(content, HttpResponse):
            return content

        template_name = self.resource.template or self.get_template_path(content)
        template = loader.get_template(template_name)
        content = template.render(RequestContext(request, dict(
                content = content,
                emitter = self,
                resource = self.resource,
            )))

        return content

    def get_template_path(self, content=None):

        if isinstance(content, Paginator):
            return 'api/paginator.%s' % self.ext

        app = ''
        name = self.resource.meta.name

        if not content:
            content = self.resource.model

        if isinstance(content, (Model, ModelBase)):
            app = content._meta.app_label
            name = content._meta.module_name

        basedir = self.resource.api.prefix if getattr(self.resource, 'api', None) else 'api'
        return op.join(
            basedir,
            self.resource.version,
            app,
            "%s.%s" % (name, self.ext)
        )


class JSONTemplateEmitter(TemplateEmitter):
    """ Emitter which serializes to JSON.
    """
    media_type = 'application/json'


class HTMLTemplateEmitter(TemplateEmitter):
    """ HTML content.
    """
    media_type = 'text/html'


class XMLTemplateEmitter(TemplateEmitter):
    """ Emitter which serializes to XML.
    """
    media_type = 'application/xml'
    template = '<?xml version="1.0" encoding="utf-8"?>\n<response success="%s" version="%s" timestamp="%s">%s</response>'

    def serialize(self, content, request=None):
        ts = int(mktime(datetime.now().timetuple()))
        if isinstance(content, HttpResponse):
            return self.template % (
                'true' if content.status_code == HTTP_200_OK else 'false',
                self.resource.version, ts, content.content)
        else:
            content = super(XMLTemplateEmitter, self).serialize(content, request=request)
            return self.template % ('true', self.resource.version, ts, content)


class JSONEmitter(BaseEmitter):

    media_type = 'application/json'

    @staticmethod
    def serialize(content, **kwargs):
        return json_dumps(content)


class XMLEmitter(BaseEmitter):

    media_type = 'application/xml'

    @staticmethod
    def serialize(content, **kwargs):
        return xml_dumps(content)



from django.core.urlresolvers import reverse
from django.db.models import Model
from django.test import TestCase, Client
from django.utils.functional import curry


class AdrestTestCase(TestCase):
    api = None

    def setUp(self):
        assert self.api, "API must be defined"
        self.client = Client()

    def reverse(self, resource, **kwargs):
        if isinstance(resource, basestring):
            url_name = resource
            assert self.api.resources.get(url_name), "Invalid resource name: %s" % url_name

        else:
            url_name = resource.meta.url_name

        kwargs = dict((k, getattr(v, "pk", v)) for k, v in kwargs.iteritems())
        name_ver = '' if not str(self.api) else '%s-' % str(self.api)
        return reverse('%s-%s%s' % (self.api.prefix, name_ver, url_name), kwargs=kwargs)

    def get_resource(self, resource, method='get', data=None, key=None, headers=None, **kwargs):
        uri = self.reverse(resource, **kwargs)
        method = getattr(self.client, method)
        if isinstance(key, Model):
            key = key.key
        headers = dict() if headers is None else headers
        headers['HTTP_AUTHORIZATION'] = key
        return method(uri, data=data or dict(), **headers)

    put_resource = curry(get_resource, method='put')
    post_resource = curry(get_resource, method='post')
    delete_resource = curry(get_resource, method='delete')



import random
import re

from django.contrib.auth.models import User
from django.core import mail
from django.core.urlresolvers import reverse
from django.http import HttpResponse
from django.test import TestCase, Client, RequestFactory
from django.views.generic import View

from .api import API as api
from .models import Author, Book
from .resourses import AuthorResource, BookPrefixResource, ArticleResource, SomeOtherResource, BookResource
from adrest.mixin.emitter import EmitterMixin
from adrest.models import Access
from adrest.tests.utils import AdrestTestCase
from adrest.utils import serializer, paginator, emitter, parser


class MixinTest(TestCase):

    def setUp(self):
        self.rf = RequestFactory()

    def test_emitter(self):
        request = self.rf.get("/")
        class Test(View, EmitterMixin):
            def get(self, request):
                content = self.emit('test')
                response = HttpResponse(content)
                return response
        test = Test()
        response = test.dispatch(request)
        self.assertContains(response, 'test')


class MetaTest(TestCase):

    def test_meta(self):
        self.assertTrue(AuthorResource.meta)
        self.assertEqual(AuthorResource.allowed_methods, (
            'GET', 'POST', 'OPTIONS', 'HEAD'
        ))
        self.assertEqual(AuthorResource.meta.name, 'author')
        self.assertEqual(AuthorResource.meta.url_name, 'author')
        self.assertEqual(AuthorResource.meta.url_regex, '^owner/$')
        self.assertEqual(AuthorResource.meta.parents, [])
        self.assertEqual(AuthorResource.meta.emitters_dict, {
            emitter.JSONEmitter.media_type: emitter.JSONEmitter,
        })
        self.assertEqual(AuthorResource.meta.emitters_types, [
            emitter.JSONEmitter.media_type,
        ])
        self.assertEqual(AuthorResource.meta.default_emitter, emitter.JSONEmitter)
        self.assertEqual(AuthorResource.meta.parsers_dict, {
            parser.FormParser.media_type: parser.FormParser,
            parser.XMLParser.media_type: parser.XMLParser,
            parser.JSONParser.media_type: parser.JSONParser,
        })
        self.assertEqual(AuthorResource.meta.default_parser, parser.FormParser)

    def test_meta_parents(self):
        self.assertEqual(AuthorResource.meta.parents, [])
        self.assertEqual(BookPrefixResource.meta.parents, [AuthorResource])
        self.assertEqual(ArticleResource.meta.parents, [AuthorResource, BookPrefixResource])

    def test_meta_name(self):
        self.assertEqual(AuthorResource.meta.name, 'author')
        self.assertEqual(BookPrefixResource.meta.name, 'book')
        self.assertEqual(SomeOtherResource.meta.name, 'someother')

    def test_meta_url_name(self):
        self.assertEqual(AuthorResource.meta.url_name, 'author')
        self.assertEqual(BookResource.meta.url_name, 'author-book')
        self.assertEqual(BookPrefixResource.meta.url_name, 'author-test-book')
        self.assertEqual(ArticleResource.meta.url_name, 'author-test-book-article')
        self.assertEqual(SomeOtherResource.meta.url_name, 'author-device-someother')

    def test_meta_url_regex(self):
        self.assertEqual(AuthorResource.meta.url_regex, '^owner/$')
        self.assertEqual(BookPrefixResource.meta.url_regex, 'owner/test/book/(?:(?P<book>[^/]+)/)?')
        self.assertEqual(ArticleResource.meta.url_regex, 'owner/book/(?P<book>[^/]+)/article/(?:(?P<article>[^/]+)/)?')
        self.assertEqual(SomeOtherResource.meta.url_regex, 'owner/device/(?P<device>[^/]+)/someother/(?:(?P<someother>[^/]+)/)?')


class ApiTest(AdrestTestCase):

    api = api

    def test_api(self):
        self.assertTrue(api.version)
        self.assertEqual(str(api), "1.0.0")
        self.assertTrue(api.urls)

    def test_urls(self):
        uri = self.reverse('test')
        self.assertEqual(uri, '/1.0.0/test/mem')


class AdrestTest(AdrestTestCase):

    api = api

    def setUp(self):
        user = User.objects.create(username='test')
        self.author = Author.objects.create(name='John', user=user)
        self.book = Book.objects.create(author=self.author, title='test', status=1)
        super(AdrestTest, self).setUp()

    def test_methods(self):
        uri = self.reverse('author')
        self.assertEqual(uri, '/1.0.0/owner')
        response = self.client.get(uri)
        self.assertContains(response, 'true')

        response = self.client.put(uri)
        self.assertContains(response, 'false', status_code=405)

        response = self.client.head(uri)
        self.assertEqual(response.status_code, 200)

    def test_owner(self):
        response = self.get_resource('author-test-book-article', book=self.book.pk, data=dict(
            author = self.author.pk
        ))
        self.assertContains(response, 'false', status_code=401)

        response = self.get_resource('author-test-book-article', key = self.author.user.accesskey_set.get(),
                book=self.book.pk, data=dict(
                    author = self.author.pk
                ))
        self.assertContains(response, 'true')

    def test_log(self):
        uri = self.reverse('author-test-book-article', book=self.book.pk)
        self.client.get(uri)
        access = Access.objects.get()
        self.assertEqual(access.uri, uri)
        self.assertEqual(access.version, str(api))

    def test_options(self):
        self.assertTrue('OPTIONS' in ArticleResource.allowed_methods)
        uri = self.reverse('author-test-book-article', book=self.book.pk)
        response = self.client.options(uri, data=dict(
            author = self.author.pk,
        ))
        self.assertContains(response, 'OK')


class ResourceTest(AdrestTestCase):

    api = api

    def setUp(self):
        super(ResourceTest, self).setUp()
        for i in range(5):
            user = User.objects.create(username='test%s' % i )
            self.author = Author.objects.create(name='author%s' % i, user=user)

        for i in range(148):
            Book.objects.create(author=self.author, title="book%s" % i, status = random.choice((1, 2, 3)))

    def test_author(self):
        response = self.get_resource('author')
        self.assertContains(response, 'count="5"')

        response = self.post_resource('author', data=dict(name = "new author", user=User.objects.create(username="new user").pk))
        self.assertContains(response, 'new author')

    def test_book(self):
        uri = self.reverse('author-test-book')
        self.assertEqual(uri, "/1.0.0/owner/test/book/")

        response = self.get_resource('author-test-book', data=dict(
            author = self.author.pk
        ))
        self.assertContains(response, 'count="%s"' % Book.objects.filter(author=self.author).count())
        self.assertContains(response, '<name>%s</name>' % self.author.name)

        response = self.post_resource('author-test-book', data=dict(
            title = "new book",
            status=2,
            author = self.author.pk))
        self.assertContains(response, '<price>0</price>')

        uri = self.reverse('author-test-book', book=1)
        uri = "%s?author=%s" % (uri, self.author.pk)
        response = self.client.put(uri, data=dict(
            price = 100
        ))
        self.assertContains(response, '<price>100</price>')

        response = self.client.delete(uri)
        self.assertContains(response, 'Book has been deleted.')

    def test_filter(self):
        uri = self.reverse('author-test-book')
        response = self.client.get(uri, data=dict(
            author = self.author.pk,
            title="book2"))
        self.assertContains(response, 'count="1"')

        response = self.client.get(uri, data=dict(
            author = self.author.pk,
            status=[1, 2, 3]))
        self.assertContains(response, 'count="%s"' % Book.objects.all().count())

        response = self.client.get(uri, data=dict(
            author = self.author.pk,
            status=[1, 3]))
        self.assertNotContains(response, '<status>2</status>')

        response = self.client.get(uri + "?title=book2&title=book3&author=%s" % self.author.pk)
        self.assertContains(response, 'count="2"')

    def test_custom(self):
        uri = self.reverse('book')
        response = self.client.get(uri)
        self.assertContains(response, 'count="%s"' % Book.objects.all().count())
        book = Book.objects.create(author=self.author, title="book", status=1)
        response = self.client.get(uri)
        self.assertContains(response, 'count="%s"' % Book.objects.all().count())

        uri = self.reverse('author-test-book-article', book=book.pk) + "?author=" + str(self.author.pk)
        response = self.client.delete(uri,
                HTTP_AUTHORIZATION=self.author.user.accesskey_set.get().key)
        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[-1].subject, '[Django] ADREST API Error (500): /1.0.0/owner/book/%s/article/' % Book.objects.all().count())

        self.assertContains(response, 'Some error', status_code=500)

        response = self.client.put(uri, HTTP_AUTHORIZATION=self.author.user.accesskey_set.get().key)
        self.assertEqual(len(mail.outbox), 2)
        self.assertEqual(mail.outbox[-1].subject, '[Django] ADREST API Error (400): /1.0.0/owner/book/%s/article/' % Book.objects.all().count())
        self.assertContains(response, 'Assertion error', status_code=400)

    def test_some_other(self):
        response = self.get_resource('test')
        self.assertContains(response, '<someother>True</someother>')
        self.assertContains(response, '<method>GET</method>')


    def test_books(self):
        """Test response Link header

        Example: </1.0.0/author/5/test/book/?page=2>; rel="next"
        """
        link_re = re.compile(r'<(?P<link>[^>]+)>\; rel=\"(?P<rel>[^\"]+)\"')

        response = self.get_resource('author-test-book',
                data=dict(author = self.author.pk))
        self.assertTrue(response.has_header("Link"))
        self.assertEquals(response["Link"], '<%s?page=2&author=5>; rel="next"' % self.reverse('author-test-book'))
        # Get objects by links on Link header
        response = self.client.get(link_re.findall(response['Link'])[0][0])

        links = link_re.findall(response['Link'])

        self.assertEquals(links[0][0], '%s?page=3&author=5' % self.reverse('author-test-book'))
        self.assertEquals(links[0][1], 'next')

        self.assertEquals(links[1][0], '%s?author=5' % self.reverse('author-test-book'))
        self.assertEquals(links[1][1], 'previous')


class AdrestMapTest(TestCase):

    def setUp(self):
        self.client = Client()

    def test_methods(self):
        uri = reverse("main-%s-map" % str(api))
        self.assertEqual(uri, "/%s/map" % api)
        response = self.client.get(uri)
        self.assertContains(response, 'MAP')
        self.assertContains(response, 'nickname')

        response = self.client.get(uri, HTTP_ACCEPT="application/json")
        self.assertNotContains(response, 'MAP')
        self.assertContains(response, '"price", {"required": false')


class SerializerTest(TestCase):

    def setUp(self):
        self.rf = RequestFactory()
        for i in range(1, 100):
            user = User.objects.create(username='test%s' % i)
            self.author = Author.objects.create(name='John %s' % i, user=user)
            self.book = Book.objects.create(author=self.author, title='test %s' % i, status=random.choice((1, 2, 3)))

    def test_json(self):
        authors = Author.objects.all()
        test = serializer.json_dumps(authors)
        self.assertTrue("main.author" in test)
        request = self.rf.get("/")
        pg = paginator.Paginator(request, authors, 10)
        test = serializer.json_dumps(pg)
        self.assertTrue("count" in test)
        books = Book.objects.all()
        test = serializer.json_dumps(books)
        self.assertTrue("author" in test)
        self.assertFalse("publisher" in test)

    def test_xml(self):
        authors = Author.objects.all()
        test = serializer.xml_dumps(authors)
        self.assertTrue("author" in test)



ROOT_URLCONF = 'main.urls'
ADMINS = ('test', 'test@test.com'),
DATABASES =  {
        'default': {
                    'ENGINE': 'django.db.backends.sqlite3',
                    'NAME': ':memory:',
                    'USER': '',
                    'PASSWORD': '',
                    'TEST_CHARSET': 'utf8',
                }
            }
INSTALLED_APPS = 'django.contrib.auth', 'django.contrib.contenttypes', 'adrest', 'tests.main', 'tests.simple'
DEBUG = TEMPLATE_DEBUG = True
CACHE_BACKEND = 'locmem://'
ADREST_ACCESS_LOG = True
ADREST_ALLOW_OPTIONS = True
ADREST_MAIL_ERRORS = 500, 400
ADREST_AUTO_CREATE_ACCESSKEY = True
TEMPLATE_CONTEXT_PROCESSORS = (
    'django.core.context_processors.static',
    'django.core.context_processors.request',
    'django.contrib.auth.context_processors.auth',
)



import base64

from django.contrib.auth import authenticate
from django.middleware.csrf import CsrfViewMiddleware


class BaseAuthenticator(object):
    """ All authenticators should extend BaseAuthenticator.
    """
    message = "Authorization required."

    def __init__(self, resource):
        self.resource = resource
        self.identifier = ''

    def authenticate(self, request=None):
        return self.get_identifier(request)

    def get_identifier(self, request=None):
        return self.identifier

    @staticmethod
    def test_rights(resources, request=None):
        return True

    @staticmethod
    def get_fields():
        return []


class AnonimousAuthenticator(BaseAuthenticator):
    " Always return true "
    __metaclass__ = type('MetaAnonimous', (type,), dict(__str__ = lambda m: "Anonymous access"))

    def get_identifier(self, request=None):
        return request.META.get('REMOTE_ADDR', 'anonymous')


class BasicAuthenticator(BaseAuthenticator):
    " Use HTTP Basic authentication "
    __metaclass__ = type('MetaAuth', (type,), dict(__str__ = lambda m: "HTTP authorization"))

    def authenticate(self, request=None):
        if 'HTTP_AUTHORIZATION' in request.META:
            auth = request.META['HTTP_AUTHORIZATION'].split()
            if len(auth) == 2 and auth[0].lower() == "basic":
                uname, passwd = base64.b64decode(auth[1]).split(':')
                user = authenticate(username=uname, password=passwd)
                if user is not None and user.is_active:
                    self.identifier = user.username
        return self.get_identifier()


class UserAuthenticator(BaseAuthenticator):
    " Django user authenticate "
    __metaclass__ = type('MetaByLogin', (type,), dict(__str__ = lambda m: "Authorization by login and password"))

    username_fieldname = 'username'
    password_fieldname = 'password'


    def authenticate(self, request=None):
        try:
            username = request.REQUEST.get(self.username_fieldname)
            password = request.REQUEST.get(self.password_fieldname)
            request.user = authenticate(username=username, password=password)
            self.identifier = request.user.username if request.user else ''
        except KeyError:
            return False
        return self.get_identifier()

    @classmethod
    def get_fields(cls):
        return [(cls.username_fieldname, dict(required=True)), (cls.password_fieldname, dict(required=True))]


class UserLoggedInAuthenticator(BaseAuthenticator):
    """ Use Djagno's built-in request session for authentication.
    """
    __metaclass__ = type('MetaBySession', (type,), dict(__str__ = lambda m: "Authorization by session"))

    def authenticate(self, request=None):
        if getattr(request, 'user', None) and request.user.is_active:
            resp = CsrfViewMiddleware().process_view(request, None, (), {})
            if resp is None:  # csrf passed
                self.identifier = request.user.username
        return self.get_identifier()

try:
    from adrest.models import AccessKey
    from django.core.exceptions import ObjectDoesNotExist


    class AccessKeyAuthenticator(BaseAuthenticator):
        " Use AccessKey identification "
        __metaclass__ = type('MetaByKey', (type,), dict(__str__ = lambda m: "Authorization by API key"))

        def authenticate(self, request=None):
            """ Authenticate user using AccessKey from HTTP Header or GET params.
            """
            try:
                access_key = request.META.get('HTTP_AUTHORIZATION') or request.REQUEST['key']
                api_key = AccessKey.objects.get(key=access_key)
                if self.test_user(api_key.user):
                    request.user = api_key.user
                    self.identifier = request.user.username
            except(KeyError, ObjectDoesNotExist):
                return False
            return self.get_identifier()

        @staticmethod
        def test_user(user):
            """ In this method you can implement User class check if you have
                several User classes with FK to AccessKey.

                With this you can use multiple AccessKeyAuthenticator classes for
                the resource and get the right class according to the test in this
                function.

                By default: doesn't care, if we find User by AccessKey, authenticate him
            """
            return True

except ImportError:
    assert True



import logging

from django.conf.urls.defaults import patterns

from .map import MapResource
from .views import ResourceView


LOG = logging.getLogger('adrest')


class Api(object):
    """ Implements a registry to tie together the various resources that make up
        an API.

        Especially useful for navigation, HATEOAS and for providing multiple
        versions of your API.
    """
    def __init__(self, version=None, api_map=True, api_prefix='api', **params):
        self.version = version
        self.prefix = api_prefix
        self.params = params
        self.resources = dict()

        if api_map:
            self.resources[MapResource.meta.url_name] = MapResource

        try:
            self.str_version = '.'.join(map(str, version or list()))
        except TypeError:
            self.str_version = str(version)

    def register(self, resource, **params):
        " Register resource subclass with API. "

        # Must be instance of ResourceView
        assert issubclass(resource, ResourceView), "%s not subclass of ResourceView" % resource

        # Fabric of resources
        params = dict(self.params, **params)
        if params:
            params['name'] = ''.join(bit for bit in resource.__name__.split('Resource') if bit).lower()
            params['__module__'] = '%s.%s' % (self.prefix, self.str_version.replace('.', '_'))
            resource = type('%s%s' % (resource.__name__, len(self.resources)), (resource,), params)

        if self.resources.get(resource.meta.url_name):
            LOG.warning("A new resource '%r' is replacing the existing record for '%s'" % (resource, self.resources.get(resource.url_name)))

        self.resources[resource.meta.url_name] = resource

    @property
    def urls(self):
        """ Provides URLconf details for the ``Api`` and all registered
            ``Resources`` beneath it.
        """

        urls = []

        for url_name in sorted(self.resources.keys()):

            resource = self.resources[url_name]
            urls.append(resource.as_url(
                api = self,
                name_prefix = '-'.join((
                            self.prefix,
                            self.str_version,
                        )).strip('-'),
                url_prefix = self.str_version
            ))

        return patterns(self.prefix, *urls)

    def __str__(self):
        return self.str_version



from .models import Task
from adrest.api import Api
from adrest.utils import emitter
from adrest.views import ResourceView


class TaskResource(ResourceView):
    model = Task
    emitters = emitter.XMLTemplateEmitter


API = Api()
API.register(TaskResource)



from adrest.utils import status
from adrest.utils.exceptions import HttpError
from adrest.utils.throttle import BaseThrottle


class ThrottleMeta(type):
    def __new__(mcs, name, bases, params):

        cls = super(ThrottleMeta, mcs).__new__(mcs, name, bases, params)
        assert issubclass(cls.throttle, BaseThrottle), "'throttle' must be subclass of BaseThrottle"
        return cls


class ThrottleMixin(object):

    __metaclass__ = ThrottleMeta

    throttle = BaseThrottle

    def throttle_check(self):
        throttle = self.throttle()
        wait = throttle.should_be_throttled(self.identifier)
        if wait:
            raise HttpError("Throttled, wait %d seconds." % wait, status=status.HTTP_503_SERVICE_UNAVAILABLE)



from .resourses import AuthorResource, BookPrefixResource, ArticleResource, SomeOtherResource, CustomResource
from adrest.api import Api
from adrest.utils.auth import AnonimousAuthenticator, AccessKeyAuthenticator, UserAuthenticator
from adrest.utils.emitter import XMLTemplateEmitter, JSONEmitter


class CustomUserAuth(UserAuthenticator):
    username_fieldname = 'nickname'


API = Api(version=(1, 0, 0), emitters=(XMLTemplateEmitter, JSONEmitter), api_prefix='main')

API.register(AuthorResource,
        authenticators=(CustomUserAuth, AnonimousAuthenticator))
API.register(BookPrefixResource)
API.register(ArticleResource, authenticators=AccessKeyAuthenticator)
API.register(CustomResource)
API.register(SomeOtherResource, url_name='test', url_regex='test/mem/$')



from django.db.models import Model
from django.db.models.fields import AutoField
from django.db.models.fields.related import ManyToManyField
from django.forms.models import ModelForm


def get_initial_from_model(model, instance):
    gen = ((f.name, f.value_from_object(instance) if instance else f.get_default()) for f in model._meta.fields if not isinstance(f, (ManyToManyField, AutoField)))
    return dict(v for v in gen if not v[1] is None)


class PartitialForm(ModelForm):

    def __init__(self, data=None, instance=None, initial=None, prefix=None, label_suffix=':', empty_permitted=False, **kwargs):
        resources = dict((k, v if not isinstance(v, Model) else v.pk) for k, v in kwargs.iteritems())
        formdata = get_initial_from_model(self._meta.model, instance)
        formdata.update(data or dict())
        formdata.update(resources)
        super(PartitialForm, self).__init__(formdata, instance=instance, prefix=prefix, label_suffix=label_suffix, empty_permitted=empty_permitted)



from .api import API
from adrest.tests.utils import AdrestTestCase


class SimpleTestCase(AdrestTestCase):
    api = API

    def test_base(self):
        uri = self.reverse('task')
        self.assertEqual(uri, '/simple/task/')
        response = self.get_resource('task')
        self.assertContains(response, 'true')



from django.conf.urls.defaults import include, patterns
from simple.api import API as simple

from .api import API as main


urlpatterns = main.urls + patterns('',
    (r'^simple/', include(simple.urls)),
)



import mimeparse

from adrest.utils import MetaOptions
from adrest.utils.emitter import JSONEmitter, BaseEmitter
from adrest.utils.tools import as_tuple


class EmitterMeta(type):

    def __new__(mcs, name, bases, params):

        params['meta'] = params.get('meta', MetaOptions())
        cls = super(EmitterMeta, mcs).__new__(mcs, name, bases, params)
        cls.emitters = as_tuple(cls.emitters)
        cls.meta.default_emitter = cls.emitters[0] if cls.emitters else None
        for e in cls.emitters:
            assert issubclass(e, BaseEmitter), "Emitter must be subclass of BaseEmitter"
            cls.meta.emitters_dict[e.media_type] = e
            cls.meta.emitters_types.append(e.media_type)
        return cls


class EmitterMixin(object):

    __metaclass__ = EmitterMeta

    emitters = JSONEmitter
    template = None

    def emit(self, content, request=None, emitter=None):
        " Serialize reponse "

        # Get emitter for request
        emitter = emitter or self._determine_emitter(request)

        # Serialize the response content
        return emitter(self).emit(content, request=request)

    def _determine_emitter(self, request):
        " Return must fine emitter for request "
        if not request:
            return self.meta.default_emitter

        if request.method == 'OPTIONS':
            return JSONEmitter

        accept = request.META.get('HTTP_ACCEPT', '*/*')
        if accept == '*/*':
            return self.meta.default_emitter

        base_format = mimeparse.best_match(self.meta.emitters_types, accept)
        return self.meta.emitters_dict.get(base_format, self.meta.default_emitter)
